/********************************************************************************************
(SAS 9.13 or higher) and (Winsteps 3.64 or higher) are required to run this program!!
The purpose of this SAS program is to import the Winsteps question parameter file, and 
to create a web-based report showing the Test Information Function (TIF) and also linking 
to other pages overlaying the Item Characteristic Curve (ICC) with Item Information Function 
(IIF) for each item. In this example, the inpute file is called 'example.que' and the output 
file is named 'report.html'. 
This SAS program utilies SAS macros. Users can simply input values into three macro variables 
in the beginning and then run the program.  
Written by Chong Ho Yu, Ph.D., Email: chonghoyu@gmail.com                          
*********************************************************************************************/  

/* Whhere is your working directory? */
%LET path = c:\IRT\;
/* what is the name of the Winsteps question parameter file */
%LET input = example.que;
/* what is the name of the Winsteps step function file */
%LET sf = example.sf;

/* import Winsteps question parameter file */
/* Classify items by difficulty levels, item-total correlation, and fitness */
data winsteps; infile "&path&input" firstobs=3;
input ORDER THRESHOLD ANCHOR TRIED RIGHT ERROR INMSQ INZSTD OUTMS 
OUTZSTD DISPL ITEM_TOTAL_CORR WEIGHT OBSMA EXPMA DISCR dummy1 $ dummy2 $ ITEMID $;

if -0.5 <= threshold <= 0.5 then difficulty = "Average  ";
else if threshold > 0.5 then difficulty = "Hard";
else if threshold < -0.5 then difficulty = "Easy"; 
if ITEM_TOTAL_CORR < 0 then correlation = "Negative";
else if ITEM_TOTAL_CORR => 0 and ITEM_TOTAL_CORR <= 0.3 then correlation = "Low";
else correlation = "Fine";
if OUTMS < 1.5 then fitness = "Fit   ";
else fitness = "Misfit";
drop weight dummy1 dummy2;
run;

/* Find the number of the last item, which will be be used in the do loop below 
for plotting ICC for each item */
data lastitem; set winsteps;
proc sort data=lastitem; by descending ITEMID; run;
data lastitem; set lastitem;
if _N_ = 1 then call symputx("lastitem", order);
run;

/* Create the theoretical theta (examinee ability) level*/
data irt; set winsteps;
do theta = -5 to 5 by 1; output; 
end;
run;

/* Compute the p value and IIF for each level of theta in each item */
data irt; set irt;
p =1/(1+EXP(-1*(theta-THRESHOLD)));
iif = p*(1-p);
run;

/* Sum all ICCs to obtain the TCC and all IIFs to obtain the TIF for each theta level */
proc summary data=irt; class theta; var p iif; output out=tcc_tif sum=tcc tif; run; 
data tcc_tif; set tcc_tif;
if _TYPE_ = 0 then delete; 
drop _TYPE_ _FREQ_;
run;

/* Import the step function file */
data sf; infile "&path&sf" firstobs=3;
input ORDER STEP STEP_THRESHOLD;
if STEP_THRESHOLD = 0 then delete; 
run;

/* Set the options for SAS Graph. If your audience does not have SAS 9.13 or above, please
change the device option from 'activex' to 'Java.' In the following DO loop, the goption 
is reset every time because ICC/IIF plot and SF plot uses different graphic option. */
%LET ICC_IIF_option =%str(
goptions reset=all device=activex;
axis1 order=(0 to 1 by .1); 
axis2 order=(-5 to 5 by 1); 
symbol1 i=join v=a c=red width=2;
symbol2 i=join v=a c=blue width=2;
);

/* Plot the ICC and IIF for each item */
%macro plot_ICC;
%DO i = 1 %to &lastitem;
ods html file="ICC&i..html" path="&path";
data tmp; set irt;
&ICC_IIF_option
	if order = &i;
	proc gplot;
	plot p*theta iif*theta / overlay vaxis=axis1 haxis=axis2;
	Title "Item Characteristic Curve and Item Information Function"; 
run;
data tmp2; set sf;
goptions reset=all device=activex;
	if order = &i;
	proc gchart; vbar step / sumvar=step_threshold discrete; 
	Title "Step Function";
	proc print noobs; var step step_threshold;
run;
ods html close;
quit;
%END;
%mend plot_ICC;

%plot_ICC

/* Create the links to ICC and IIF plots in the final report */
data output; set winsteps;
temp="'ICC"||order||".html' target=new>";
temp2=compress(temp);
ID = "<a href="||temp2||ITEMID||"</a>";
drop temp temp2;
run;

/* Plot the TIF in the final report */
ods html file="report.html" path="&path";
proc print noobs data=output; 
var ID THRESHOLD TRIED RIGHT ERROR INMSQ INZSTD OUTMS OUTZSTD ITEM_TOTAL_CORR OBSMA EXPMA DISCR; 
title "Winsteps Item Parameters with links to ICC/IIF"; 
data tcc_tif; set tcc_tif;
goptions reset=all device=activex;
symbol1 i=join v=a c=green width=2;
symbol2 i=join v=a c=purple width=2;
proc gplot data=tcc_tif;
        plot tif*theta tcc*theta / overlay;
	    Title "Test Information Function and Test Characteristic Curve";
proc gchart data=winsteps;
		hbar ITEMID /sumvar=THRESHOLD subgroup=difficulty;
		Title "Overview of item difficulty: below -0.5=very easy, above 0.5 = very hard, else = average";
proc gchart data=winsteps;
		hbar ITEMID /sumvar=ITEM_TOTAL_CORR subgroup=correlation;
		Title "Diagnosis of item-total correlation (Negative is bad, below .3 is low)";
proc gchart data=winsteps;
		hbar ITEMID /sumvar=OUTMS subgroup=fitness;
		Title "Detection of misfit items (Outfit Mean squares > 1.5 is bad)"; run;
run;
ods html close;
quit;

